@@ -11,7 +11,7 @@
 
 struct clish_sym_s {
 	char *name; /* Symbol name */
-	void *func; /* Function address */
+	const void *func; /* Function address */
 	int type; /* Function type */
 	clish_sym_api_e api; /* Function API */
 	bool_t permanent; /* If permanent the dry-run can't switch it off */
@@ -29,4 +29,5 @@ struct clish_plugin_s {
 	clish_plugin_init_t *init; /* Init function (constructor) != NULL */
 	clish_plugin_fini_t *fini; /* Fini function (destructor) */
 	bool_t rtld_global; /* RTLD_GLOBAL flag for dlopen() */
+	void *userdata; /* Arbitrary userdata to pass to ini and fini functions */
 };
