@@ -13,43 +13,140 @@
 #include <limits.h>
 #include <stdio.h>
 
-/*---------------------------------------------------------
- * PRIVATE METHODS
- *--------------------------------------------------------- */
-static char *clish_ptype_select__get_name(const clish_ptype_t * this,
-	unsigned index)
+/*--------------------------------------------------------- */
+int clish_ptype_compare(const void *first, const void *second)
 {
-	char *result = NULL;
-	const char *arg = lub_argv__get_arg(this->u.select.items, index);
-	if (arg) {
-		size_t name_len = strlen(arg);
-		const char *lbrk = strchr(arg, '(');
-		if (lbrk)
-			name_len = (size_t) (lbrk - arg);
-		result = lub_string_dupn(arg, name_len);
-	}
-	return result;
+	const clish_ptype_t *f = (const clish_ptype_t *)first;
+	const clish_ptype_t *s = (const clish_ptype_t *)second;
+
+	return strcmp(f->name, s->name);
 }
 
 /*--------------------------------------------------------- */
-static char *clish_ptype_select__get_value(const clish_ptype_t * this,
-	unsigned index)
+static void clish_ptype_init(clish_ptype_t * this,
+	const char *name, const char *text, const char *pattern,
+	clish_ptype_method_e method, clish_ptype_preprocess_e preprocess)
 {
-	char *result = NULL;
-	const char *arg = lub_argv__get_arg(this->u.select.items, index);
-	if (arg) {
-		const char *lbrk = strchr(arg, '(');
-		const char *rbrk = strchr(arg, ')');
-		const char *value = arg;
-		size_t value_len = strlen(arg);
-		if (lbrk) {
-			value = lbrk + 1;
-			if (rbrk)
-				value_len = (size_t) (rbrk - value);
-		}
-		result = lub_string_dupn(value, value_len);
+	assert(this);
+	assert(name);
+	this->name = lub_string_dup(name);
+	this->text = NULL;
+	this->completion = NULL;
+	this->pattern = NULL;
+	this->preprocess = preprocess;
+	this->range = NULL;
+	this->action = clish_action_new();
+	// PTYPE's ACTION must be 'permanent' i.e. it must be executed whenever
+	// it's a dryrun klish mode or not. Because argument checking is always
+	// needed.
+	clish_action__set_permanent(this->action, BOOL_TRUE);
+
+	if (pattern) {
+		/* set the pattern for this type */
+		clish_ptype__set_pattern(this, pattern, method);
+	} else {
+		// Only method="code" doesn't need a pattern
+		this->method = CLISH_PTYPE_METHOD_CODE;
 	}
-	return result;
+	
+	/* set the help text for this type */
+	if (text)
+		clish_ptype__set_text(this, text);
+}
+
+/*--------------------------------------------------------- */
+static void clish_ptype_fini(clish_ptype_t * this)
+{
+	if (this->pattern) {
+		switch (this->method) {
+		case CLISH_PTYPE_METHOD_REGEXP:
+			if (this->u.regex.is_compiled)
+				regfree(&this->u.regex.re);
+			break;
+		case CLISH_PTYPE_METHOD_INTEGER:
+		case CLISH_PTYPE_METHOD_UNSIGNEDINTEGER:
+			break;
+		case CLISH_PTYPE_METHOD_SELECT:
+			lub_argv_delete(this->u.select.items);
+			break;
+		case CLISH_PTYPE_METHOD_CODE:
+			break;
+		default:
+			break;
+		}
+	}
+
+	lub_string_free(this->name);
+	lub_string_free(this->text);
+	lub_string_free(this->pattern);
+	lub_string_free(this->range);
+	lub_string_free(this->completion);
+	clish_action_delete(this->action);
+}
+
+/*--------------------------------------------------------- */
+clish_ptype_t *clish_ptype_new(const char *name,
+	const char *help, const char *pattern,
+	clish_ptype_method_e method, clish_ptype_preprocess_e preprocess)
+{
+	clish_ptype_t *this = malloc(sizeof(clish_ptype_t));
+
+	if (this)
+		clish_ptype_init(this, name, help, pattern, method, preprocess);
+	return this;
+}
+
+/*--------------------------------------------------------- */
+void clish_ptype_free(void *data)
+{
+	clish_ptype_t *this = (clish_ptype_t *)data;
+	clish_ptype_fini(this);
+	free(this);
+}
+
+/*--------------------------------------------------------- */
+static char *clish_ptype_select__get_name(const clish_ptype_t *this,
+	unsigned int index)
+{
+	char *res = NULL;
+	size_t name_len;
+	const char *arg = lub_argv__get_arg(this->u.select.items, index);
+
+	if (!arg)
+		return NULL;
+	name_len = strlen(arg);
+	const char *lbrk = strchr(arg, '(');
+	if (lbrk)
+		name_len = (size_t) (lbrk - arg);
+	res = lub_string_dupn(arg, name_len);
+
+	return res;
+}
+
+/*--------------------------------------------------------- */
+static char *clish_ptype_select__get_value(const clish_ptype_t *this,
+	unsigned int index)
+{
+	char *res = NULL;
+	const char *lbrk, *rbrk, *value;
+	size_t value_len;
+	const char *arg = lub_argv__get_arg(this->u.select.items, index);
+
+	if (!arg)
+		return NULL;
+
+	lbrk = strchr(arg, '(');
+	rbrk = strchr(arg, ')');
+	value = arg;
+	value_len = strlen(arg);
+	if (lbrk) {
+		value = lbrk + 1;
+		if (rbrk)
+			value_len = (size_t) (rbrk - value);
+	}
+	res = lub_string_dupn(value, value_len);
+
+	return res;
 }
 
 /*--------------------------------------------------------- */
@@ -60,11 +157,11 @@ static void clish_ptype__set_range(clish_ptype_t * this)
 	/* Now set up the range values */
 	switch (this->method) {
 	/*------------------------------------------------- */
-	case CLISH_PTYPE_REGEXP:
+	case CLISH_PTYPE_METHOD_REGEXP:
 		/* Nothing more to do */
 		break;
 	/*------------------------------------------------- */
-	case CLISH_PTYPE_INTEGER:
+	case CLISH_PTYPE_METHOD_INTEGER:
 		/* Setup the integer range */
 		snprintf(tmp, sizeof(tmp), "%d..%d",
 			this->u.integer.min, this->u.integer.max);
@@ -72,7 +169,7 @@ static void clish_ptype__set_range(clish_ptype_t * this)
 		this->range = lub_string_dup(tmp);
 		break;
 	/*------------------------------------------------- */
-	case CLISH_PTYPE_UNSIGNEDINTEGER:
+	case CLISH_PTYPE_METHOD_UNSIGNEDINTEGER:
 		/* Setup the unsigned integer range */
 		snprintf(tmp, sizeof(tmp), "%u..%u",
 			(unsigned int)this->u.integer.min,
@@ -81,7 +178,7 @@ static void clish_ptype__set_range(clish_ptype_t * this)
 		this->range = lub_string_dup(tmp);
 		break;
 	/*------------------------------------------------- */
-	case CLISH_PTYPE_SELECT:
+	case CLISH_PTYPE_METHOD_SELECT:
 	{
 		/* Setup the selection values to the help text */
 		unsigned int i;
@@ -99,69 +196,46 @@ static void clish_ptype__set_range(clish_ptype_t * this)
 		break;
 	}
 	/*------------------------------------------------- */
+	case CLISH_PTYPE_METHOD_CODE:
+		// Nothing to do
+		break;
+	/*------------------------------------------------- */
+	default:
+		break;
+	/*------------------------------------------------- */
 	}
 }
 
-/*---------------------------------------------------------
- * PUBLIC META FUNCTIONS
- *--------------------------------------------------------- */
-int clish_ptype_bt_compare(const void *clientnode, const void *clientkey)
-{
-	const clish_ptype_t *this = clientnode;
-	const char *key = clientkey;
-
-	return strcmp(this->name, key);
-}
-
-/*-------------------------------------------------------- */
-void clish_ptype_bt_getkey(const void *clientnode, lub_bintree_key_t * key)
-{
-	const clish_ptype_t *this = clientnode;
-
-	/* fill out the opaque key */
-	strcpy((char *)key, this->name);
-}
-
-/*--------------------------------------------------------- */
-size_t clish_ptype_bt_offset(void)
-{
-	return offsetof(clish_ptype_t, bt_node);
-}
-
 /*--------------------------------------------------------- */
 static const char *method_names[] = {
 	"regexp",
 	"integer",
 	"unsignedInteger",
-	"select"
+	"select",
+	"code"
 };
 
 /*--------------------------------------------------------- */
-const char *clish_ptype_method__get_name(clish_ptype_method_e method)
+const char *clish_ptype__get_method_name(clish_ptype_method_e method)
 {
-	unsigned int max_method = sizeof(method_names) / sizeof(char *);
-
-	if (method >= max_method)
+	if (method >= CLISH_PTYPE_METHOD_MAX)
 		return NULL;
 	return method_names[method];
 }
 
 /*--------------------------------------------------------- */
+/* Return value of CLISH_PTYPE_METHOD_MAX indicates an illegal method */
 clish_ptype_method_e clish_ptype_method_resolve(const char *name)
 {
-	clish_ptype_method_e result = CLISH_PTYPE_REGEXP;
-	if (NULL != name) {
-		unsigned i;
-		for (i = 0; i < CLISH_PTYPE_SELECT + 1; i++) {
-			if (0 == strcmp(name, method_names[i])) {
-				result = (clish_ptype_method_e) i;
-				break;
-			}
-		}
-		/* error for incorrect type spec */
-		assert(i <= CLISH_PTYPE_SELECT);
+	unsigned int i;
+
+	if (NULL == name)
+		return CLISH_PTYPE_METHOD_REGEXP;
+	for (i = 0; i < CLISH_PTYPE_METHOD_MAX; i++) {
+		if (!strcmp(name, method_names[i]))
+			break;
 	}
-	return result;
+	return (clish_ptype_method_e)i;
 }
 
 /*--------------------------------------------------------- */
@@ -172,43 +246,39 @@ static const char *preprocess_names[] = {
 };
 
 /*--------------------------------------------------------- */
-const char *clish_ptype_preprocess__get_name(
-	clish_ptype_preprocess_e preprocess)
+const char *clish_ptype__get_preprocess_name(clish_ptype_preprocess_e preprocess)
 {
+	if (preprocess >= CLISH_PTYPE_PRE_MAX)
+		return NULL;
+
 	return preprocess_names[preprocess];
 }
 
 /*--------------------------------------------------------- */
+/* Return value of CLISH_PTYPE_PRE_MAX indicates an illegal method */
 clish_ptype_preprocess_e clish_ptype_preprocess_resolve(const char *name)
 {
-	clish_ptype_preprocess_e result = CLISH_PTYPE_NONE;
-	if (name) {
-		unsigned i;
-		for (i = 0; i < CLISH_PTYPE_TOLOWER + 1; i++) {
-			if (0 == strcmp(name, preprocess_names[i])) {
-				result = (clish_ptype_preprocess_e) i;
-				break;
-			}
-		}
-		/* error for incorrect type spec */
-		assert((clish_ptype_preprocess_e) i <= CLISH_PTYPE_TOLOWER);
-	}
-	return result;
-}
+	unsigned int i;
 
-/*---------------------------------------------------------
- * PUBLIC METHODS
- *--------------------------------------------------------- */
+	if (NULL == name)
+		return CLISH_PTYPE_PRE_NONE;
+	for (i = 0; i < CLISH_PTYPE_PRE_MAX; i++) {
+		if (!strcmp(name, preprocess_names[i]))
+			break;
+	}
+
+	return (clish_ptype_preprocess_e)i;
+}
 
 /*--------------------------------------------------------- */
 void clish_ptype_word_generator(clish_ptype_t * this,
 	lub_argv_t *matches, const char *text)
 {
 	char *result = NULL;
-	unsigned i = 0;
+	unsigned int i = 0;
 
-	/* Another ptypes has no completions */
-	if (this->method != CLISH_PTYPE_SELECT)
+	/* Only METHOD_SELECT has completions */
+	if (this->method != CLISH_PTYPE_METHOD_SELECT)
 		return;
 
 	/* First of all simply try to validate the result */
@@ -229,7 +299,7 @@ void clish_ptype_word_generator(clish_ptype_t * this,
 }
 
 /*--------------------------------------------------------- */
-static char *clish_ptype_validate_or_translate(const clish_ptype_t * this,
+static char *clish_ptype_validate_or_translate(clish_ptype_t * this,
 	const char *text, bool_t translate)
 {
 	char *result = lub_string_dup(text);
@@ -237,24 +307,20 @@ static char *clish_ptype_validate_or_translate(const clish_ptype_t * this,
 
 	switch (this->preprocess) {
 	/*----------------------------------------- */
-	case CLISH_PTYPE_NONE:
+	case CLISH_PTYPE_PRE_NONE:
 		break;
 	/*----------------------------------------- */
-	case CLISH_PTYPE_TOUPPER:
+	case CLISH_PTYPE_PRE_TOUPPER:
 	{
 		char *p = result;
 		while (*p) {
-			/*lint -e155 Ignoring { }'ed sequence within an expression, 0 assumed 
-			 * MACRO implementation uses braces to prevent multiple increments
-			 * when called.
-			 */
 			*p = lub_ctype_toupper(*p);
 			p++;
 		}
 		break;
 	}
 	/*----------------------------------------- */
-	case CLISH_PTYPE_TOLOWER:
+	case CLISH_PTYPE_PRE_TOLOWER:
 	{
 		char *p = result;
 		while (*p) {
@@ -264,30 +330,37 @@ static char *clish_ptype_validate_or_translate(const clish_ptype_t * this,
 		break;
 	}
 	/*----------------------------------------- */
+	default:
+		break;
 	}
-	/*
-	 * now validate according the specified method 
-	 */
+
+	/* Validate according the specified method */
 	switch (this->method) {
 	/*------------------------------------------------- */
-	case CLISH_PTYPE_REGEXP:
-		/* test the regular expression against the string */
-		/*lint -e64 Type mismatch (arg. no. 4) */
-		/*
-		 * lint seems to equate regmatch_t[] as being of type regmatch_t !
-		 */
-		if (0 != regexec(&this->u.regexp, result, 0, NULL, 0)) {
+	case CLISH_PTYPE_METHOD_REGEXP:
+		/* Lazy compilation of the regular expression */
+		if (!this->u.regex.is_compiled) {
+			if (regcomp(&this->u.regex.re, this->pattern,
+				REG_NOSUB | REG_EXTENDED)) {
+				lub_string_free(result);
+				result = NULL;
+				break;
+			}
+			this->u.regex.is_compiled = BOOL_TRUE;
+		}
+
+		if (regexec(&this->u.regex.re, result, 0, NULL, 0)) {
 			lub_string_free(result);
 			result = NULL;
 		}
-		/*lint +e64 */
 		break;
 	/*------------------------------------------------- */
-	case CLISH_PTYPE_INTEGER:
+	case CLISH_PTYPE_METHOD_INTEGER:
 	{
 		/* first of all check that this is a number */
 		bool_t ok = BOOL_TRUE;
 		const char *p = result;
+		int value = 0;
 
 		if (*p == '-')
 			p++;
@@ -297,206 +370,122 @@ static char *clish_ptype_validate_or_translate(const clish_ptype_t * this,
 				break;
 			}
 		}
-		if (BOOL_TRUE == ok) {
-			/* convert and check the range */
-			int value = 0;
-			if ((lub_conv_atoi(result, &value, 0) < 0) ||
-				(value < this->u.integer.min) ||
-				(value > this->u.integer.max)) {
-				lub_string_free(result);
-				result = NULL;
-			}
-		} else {
+		if (BOOL_FALSE == ok) {
 			lub_string_free(result);
 			result = NULL;
+			break;
+		}
+		/* Convert to number and check the range */
+		if ((lub_conv_atoi(result, &value, 0) < 0) ||
+			(value < this->u.integer.min) ||
+			(value > this->u.integer.max)) {
+			lub_string_free(result);
+			result = NULL;
+			break;
 		}
 		break;
 	}
 	/*------------------------------------------------- */
-	case CLISH_PTYPE_UNSIGNEDINTEGER:
+	case CLISH_PTYPE_METHOD_UNSIGNEDINTEGER:
 	{
 		/* first of all check that this is a number */
 		bool_t ok = BOOL_TRUE;
 		const char *p = result;
+		unsigned int value = 0;
 		while (p && *p) {
 			if (!lub_ctype_isdigit(*p++)) {
 				ok = BOOL_FALSE;
 				break;
 			}
 		}
-		if (BOOL_TRUE == ok) {
-			/* convert and check the range */
-			unsigned int value = 0;
-			if ((lub_conv_atoui(result, &value, 0) < 0) ||
-				(value < (unsigned)this->u.integer.min) ||
-				(value > (unsigned)this->u.integer.max)) {
-				lub_string_free(result);
-				result = NULL;
-			}
-		} else {
+		if (BOOL_FALSE == ok) {
 			lub_string_free(result);
 			result = NULL;
+			break;
+		}
+		/* Convert to number and check the range */
+		if ((lub_conv_atoui(result, &value, 0) < 0) ||
+			(value < (unsigned)this->u.integer.min) ||
+			(value > (unsigned)this->u.integer.max)) {
+			lub_string_free(result);
+			result = NULL;
+			break;
 		}
 		break;
 	}
 	/*------------------------------------------------- */
-	case CLISH_PTYPE_SELECT:
+	case CLISH_PTYPE_METHOD_SELECT:
 	{
-		unsigned i;
-		for (i = 0; i < lub_argv__get_count(this->u.select.items);
-			i++) {
+		unsigned int i;
+		for (i = 0; i < lub_argv__get_count(this->u.select.items); i++) {
 			char *name = clish_ptype_select__get_name(this, i);
 			char *value = clish_ptype_select__get_value(this, i);
 			int tmp = lub_string_nocasecmp(result, name);
-			lub_string_free((BOOL_TRUE ==
-				translate) ? name : value);
+			lub_string_free((BOOL_TRUE == translate) ? name : value);
 			if (0 == tmp) {
 				lub_string_free(result);
-				result = ((BOOL_TRUE ==
-					translate) ? value : name);
+				result = ((BOOL_TRUE == translate) ? value : name);
 				break;
-			} else {
-				lub_string_free((BOOL_TRUE ==
-					translate) ? value : name);
 			}
+			lub_string_free((BOOL_TRUE == translate) ? value : name);
 		}
 		if (i == lub_argv__get_count(this->u.select.items)) {
 			/* failed to find a match */
 			lub_string_free(result);
 			result = NULL;
+			break;
 		}
 		break;
 	}
 	/*------------------------------------------------- */
+	default:
+		break;
 	}
 	return (char *)result;
 }
 
 /*--------------------------------------------------------- */
-static void clish_ptype_init(clish_ptype_t * this,
-	const char *name, const char *text, const char *pattern,
-	clish_ptype_method_e method, clish_ptype_preprocess_e preprocess)
-{
-	assert(name);
-	this->name = lub_string_dup(name);
-	this->text = NULL;
-	this->pattern = NULL;
-	this->preprocess = preprocess;
-	this->range = NULL;
-
-	/* Be a good binary tree citizen */
-	lub_bintree_node_init(&this->bt_node);
-
-	if (pattern) {
-		/* set the pattern for this type */
-		clish_ptype__set_pattern(this, pattern, method);
-	} else {
-		/* The method is regexp by default */
-		this->method = CLISH_PTYPE_REGEXP;
-	}
-	
-	/* set the help text for this type */
-	if (text)
-		clish_ptype__set_text(this, text);
-}
-
-/*--------------------------------------------------------- */
-char *clish_ptype_validate(const clish_ptype_t * this, const char *text)
+char *clish_ptype_validate(clish_ptype_t * this, const char *text)
 {
 	return clish_ptype_validate_or_translate(this, text, BOOL_FALSE);
 }
 
 /*--------------------------------------------------------- */
-char *clish_ptype_translate(const clish_ptype_t * this, const char *text)
+char *clish_ptype_translate(clish_ptype_t * this, const char *text)
 {
 	return clish_ptype_validate_or_translate(this, text, BOOL_TRUE);
 }
 
-/*--------------------------------------------------------- */
-clish_ptype_t *clish_ptype_new(const char *name,
-	const char *help, const char *pattern,
-	clish_ptype_method_e method, clish_ptype_preprocess_e preprocess)
-{
-	clish_ptype_t *this = malloc(sizeof(clish_ptype_t));
-
-	if (this)
-		clish_ptype_init(this, name, help, pattern, method, preprocess);
-	return this;
-}
+CLISH_GET_STR(ptype, name);
+CLISH_SET_STR_ONCE(ptype, text);
+CLISH_GET_STR(ptype, text);
+CLISH_SET_STR_ONCE(ptype, completion);
+CLISH_GET_STR(ptype, completion);
+CLISH_SET_ONCE(ptype, clish_ptype_preprocess_e, preprocess);
+CLISH_GET_STR(ptype, range);
+CLISH_GET(ptype, clish_action_t *, action);
+CLISH_GET(ptype, clish_ptype_method_e, method);
 
 /*--------------------------------------------------------- */
-static void clish_ptype_fini(clish_ptype_t * this)
-{
-	if (this->pattern) {
-		switch (this->method) {
-		case CLISH_PTYPE_REGEXP:
-			regfree(&this->u.regexp);
-			break;
-		case CLISH_PTYPE_INTEGER:
-		case CLISH_PTYPE_UNSIGNEDINTEGER:
-			break;
-		case CLISH_PTYPE_SELECT:
-			lub_argv_delete(this->u.select.items);
-			break;
-		}
-	}
-
-	lub_string_free(this->name);
-	this->name = NULL;
-	lub_string_free(this->text);
-	this->text = NULL;
-	lub_string_free(this->pattern);
-	this->pattern = NULL;
-	lub_string_free(this->range);
-	this->range = NULL;
-}
-
-/*--------------------------------------------------------- */
-void clish_ptype_delete(clish_ptype_t * this)
-{
-	clish_ptype_fini(this);
-	free(this);
-}
-
-/*--------------------------------------------------------- */
-const char *clish_ptype__get_name(const clish_ptype_t * this)
-{
-	return (const char *)this->name;
-}
-
-/*--------------------------------------------------------- */
-const char *clish_ptype__get_text(const clish_ptype_t * this)
-{
-	return (const char *)this->text;
-}
-
-/*--------------------------------------------------------- */
-void
-clish_ptype__set_pattern(clish_ptype_t * this,
-			 const char *pattern, clish_ptype_method_e method)
+void clish_ptype__set_pattern(clish_ptype_t * this,
+	const char *pattern, clish_ptype_method_e method)
 {
 	assert(NULL == this->pattern);
 	this->method = method;
 
 	switch (this->method) {
 	/*------------------------------------------------- */
-	case CLISH_PTYPE_REGEXP:
+	case CLISH_PTYPE_METHOD_REGEXP:
 	{
-		int result;
-
-		/* only the expression is allowed */
 		lub_string_cat(&this->pattern, "^");
 		lub_string_cat(&this->pattern, pattern);
 		lub_string_cat(&this->pattern, "$");
-
-		/* compile the regular expression for later use */
-		result = regcomp(&this->u.regexp, this->pattern,
-			REG_NOSUB | REG_EXTENDED);
-		assert(0 == result);
+		/* Use lazy mechanism to compile regular expressions */
+		this->u.regex.is_compiled = BOOL_FALSE;
 		break;
 	}
 	/*------------------------------------------------- */
-	case CLISH_PTYPE_INTEGER:
+	case CLISH_PTYPE_METHOD_INTEGER:
 		/* default the range to that of an integer */
 		this->u.integer.min = INT_MIN;
 		this->u.integer.max = INT_MAX;
@@ -506,7 +495,7 @@ clish_ptype__set_pattern(clish_ptype_t * this,
 			&this->u.integer.min, &this->u.integer.max);
 		break;
 	/*------------------------------------------------- */
-	case CLISH_PTYPE_UNSIGNEDINTEGER:
+	case CLISH_PTYPE_METHOD_UNSIGNEDINTEGER:
 		/* default the range to that of an unsigned integer */
 		this->u.integer.min = 0;
 		this->u.integer.max = (int)UINT_MAX;
@@ -517,38 +506,19 @@ clish_ptype__set_pattern(clish_ptype_t * this,
 			(unsigned int *)&this->u.integer.max);
 		break;
 	/*------------------------------------------------- */
-	case CLISH_PTYPE_SELECT:
+	case CLISH_PTYPE_METHOD_SELECT:
 		this->pattern = lub_string_dup(pattern);
 		/* store a vector of item descriptors */
 		this->u.select.items = lub_argv_new(this->pattern, 0);
 		break;
 	/*------------------------------------------------- */
+	case CLISH_PTYPE_METHOD_CODE:
+		// Nothing to do
+		break;
+	/*------------------------------------------------- */
+	default:
+		break;
 	}
 	/* now set up the range details */
 	clish_ptype__set_range(this);
 }
-
-/*--------------------------------------------------------- */
-void clish_ptype__set_text(clish_ptype_t * this, const char *text)
-{
-	assert(!this->text);
-	this->text = lub_string_dup(text);
-
-}
-
-/*--------------------------------------------------------- */
-void
-clish_ptype__set_preprocess(clish_ptype_t * this,
-	clish_ptype_preprocess_e preprocess)
-{
-	assert(!this->preprocess);
-	this->preprocess = preprocess;
-}
-
-/*--------------------------------------------------------- */
-const char *clish_ptype__get_range(const clish_ptype_t * this)
-{
-	return (const char *)this->range;
-}
-
-/*--------------------------------------------------------- */
