@@ -92,7 +92,7 @@ int clish_shell_execute(clish_context_t *context, char **out)
 	clish_shell_t *this = clish_context__get_shell(context);
 	const clish_command_t *cmd = clish_context__get_cmd(context);
 	int result = 0;
-	char *lock_path = clish_shell__get_lockfile(this);
+	const char *lock_path = clish_shell__get_lockfile(this);
 	int lock_fd = -1;
 	clish_view_t *cur_view = clish_shell__get_view(this);
 	unsigned int saved_wdog_timeout = this->wdog_timeout;
@@ -113,7 +113,7 @@ int clish_shell_execute(clish_context_t *context, char **out)
 	}
 
 	/* Lock the lockfile */
-	if (lock_path && clish_command__get_lock(cmd)) {
+	if (lock_path && clish_action__get_lock(clish_command__get_action(cmd))) {
 		lock_fd = clish_shell_lock(lock_path);
 		if (-1 == lock_fd) {
 			result = -1;
@@ -123,8 +123,7 @@ int clish_shell_execute(clish_context_t *context, char **out)
 
 	/* Execute ACTION */
 	clish_context__set_action(context, clish_command__get_action(cmd));
-	result = clish_shell_exec_action(context, out,
-		clish_command__get_interrupt(cmd));
+	result = clish_shell_exec_action(context, out);
 
 	/* Call config callback */
 	if (!result)
@@ -138,6 +137,21 @@ int clish_shell_execute(clish_context_t *context, char **out)
 		lub_string_free(full_line);
 	}
 
+	if (clish_shell__get_canon_out(this) &&
+		!clish_command__get_internal(cmd)) {
+		char *space = NULL;
+		char *full_line = clish_shell__get_full_line(context);
+		if (this->depth > 0) {
+			space = malloc(this->depth + 1);
+			memset(space, ' ', this->depth);
+			space[this->depth] = '\0';
+		}
+		printf("%s%s\n", space ? space : "", full_line);
+		lub_string_free(full_line);
+		if (space)
+			free(space);
+	}
+
 	/* Unlock the lockfile */
 	if (lock_fd != -1)
 		clish_shell_unlock(lock_fd);
@@ -211,10 +225,11 @@ static int clish_shell_exec_oaction(clish_hook_oaction_fn_t func,
 		struct iovec *iov;
 		const int rsize = CLISH_STDOUT_CHUNK; /* Read chunk size */
 		size_t cur_size = 0;
+		ssize_t r = 0;
 
 		close(pipe1[1]);
 		close(pipe2[0]);
-		l = lub_list_new(NULL);
+		l = lub_list_new(NULL, NULL);
 
 		/* Read the result of script execution */
 		while (1) {
@@ -244,14 +259,14 @@ static int clish_shell_exec_oaction(clish_hook_oaction_fn_t func,
 			iov = lub_list_node__get_data(node);
 			lub_list_del(l, node);
 			lub_list_node_free(node);
-			write(pipe2[1], iov->iov_base, iov->iov_len);
+			r = write(pipe2[1], iov->iov_base, iov->iov_len);
 			free(iov->iov_base);
 			free(iov);
 		}
 		close(pipe2[1]);
 
 		lub_list_free(l);
-		_exit(0);
+		_exit(r < 0 ? 1 : 0);
 	}
 
 	real_stdout = dup(STDOUT_FILENO);
@@ -272,7 +287,7 @@ static int clish_shell_exec_oaction(clish_hook_oaction_fn_t func,
 	konf_buf_delete(buf);
 	close(pipe2[0]);
 	/* Wait for the stdout-grabber process */
-	waitpid(cpid, NULL, 0);
+	while (waitpid(cpid, NULL, 0) != cpid);
 
 	return result;
 
@@ -283,14 +298,15 @@ stdout_error:
 }
 
 /*----------------------------------------------------------- */
-int clish_shell_exec_action(clish_context_t *context, char **out, bool_t intr)
+int clish_shell_exec_action(clish_context_t *context, char **out)
 {
 	int result = -1;
-	clish_sym_t *sym;
+	const clish_sym_t *sym;
 	char *script;
-	void *func = NULL; /* We don't know the func API at this time */
+	const void *func = NULL; /* We don't know the func API at this time */
 	const clish_action_t *action = clish_context__get_action(context);
 	clish_shell_t *shell = clish_context__get_shell(context);
+	bool_t intr = clish_action__get_interrupt(action);
 	/* Signal vars */
 	struct sigaction old_sigint, old_sigquit, old_sighup;
 	struct sigaction sa;
@@ -298,7 +314,9 @@ int clish_shell_exec_action(clish_context_t *context, char **out, bool_t intr)
 
 	if (!(sym = clish_action__get_builtin(action)))
 		return 0;
-	if (shell->dryrun && !clish_sym__get_permanent(sym))
+	if (shell->dryrun &&
+		!clish_sym__get_permanent(sym) &&
+		!clish_action__get_permanent(action))
 		return 0;
 	if (!(func = clish_sym__get_func(sym))) {
 		fprintf(stderr, "Error: Default ACTION symbol is not specified.\n");
@@ -364,11 +382,11 @@ int clish_shell_exec_action(clish_context_t *context, char **out, bool_t intr)
 }
 
 /*----------------------------------------------------------- */
-void *clish_shell_check_hook(const clish_context_t *clish_context, int type)
+const void *clish_shell_check_hook(const clish_context_t *clish_context, int type)
 {
 	clish_sym_t *sym;
 	clish_shell_t *shell = clish_context__get_shell(clish_context);
-	void *func;
+	const void *func;
 
 	if (!(sym = shell->hooks[type]))
 		return NULL;
@@ -418,33 +436,13 @@ char *clish_shell_mkfifo(clish_shell_t * this, char *name, size_t n)
 /*----------------------------------------------------------- */
 int clish_shell_rmfifo(clish_shell_t * this, const char *name)
 {
+	this = this; // Happy compiler
 	return unlink(name);
 }
 
-/*-------------------------------------------------------- */
-void clish_shell__set_log(clish_shell_t *this, bool_t log)
-{
-	assert(this);
-	this->log = log;
-}
-
-/*-------------------------------------------------------- */
-bool_t clish_shell__get_log(const clish_shell_t *this)
-{
-	assert(this);
-	return this->log;
-}
-
-/*-------------------------------------------------------- */
-void clish_shell__set_dryrun(clish_shell_t *this, bool_t dryrun)
-{
-	this->dryrun = dryrun;
-}
-
-/*-------------------------------------------------------- */
-bool_t clish_shell__get_dryrun(const clish_shell_t *this)
-{
-	return this->dryrun;
-}
-
-/*----------------------------------------------------------- */
+CLISH_SET(shell, bool_t, log);
+CLISH_GET(shell, bool_t, log);
+CLISH_SET(shell, bool_t, dryrun);
+CLISH_GET(shell, bool_t, dryrun);
+CLISH_SET(shell, bool_t, canon_out);
+CLISH_GET(shell, bool_t, canon_out);
