@@ -23,15 +23,11 @@ static void clish_shell_init(clish_shell_t * this,
 	int i;
 	char template[PATH_MAX];
 
-	/* initialise the tree of views */
-	lub_bintree_init(&this->view_tree,
-		clish_view_bt_offset(),
-		clish_view_bt_compare, clish_view_bt_getkey);
+	/* Initialise VIEW list */
+	this->view_tree = lub_list_new(clish_view_compare, clish_view_delete);
 
-	/* initialise the tree of ptypes */
-	lub_bintree_init(&this->ptype_tree,
-		clish_ptype_bt_offset(),
-		clish_ptype_bt_compare, clish_ptype_bt_getkey);
+	/* Init PTYPE list */
+	this->ptype_tree = lub_list_new(clish_ptype_compare, clish_ptype_free);
 
 	/* initialise the tree of vars */
 	lub_bintree_init(&this->var_tree,
@@ -39,14 +35,13 @@ static void clish_shell_init(clish_shell_t * this,
 		clish_var_bt_compare, clish_var_bt_getkey);
 
 	/* Initialize plugin list */
-	this->plugins = lub_list_new(NULL);
+	this->plugins = lub_list_new(NULL, clish_plugin_free);
 
 	/* Initialise the list of unresolved (yet) symbols */
-	this->syms = lub_list_new(clish_sym_compare);
+	this->syms = lub_list_new(clish_sym_compare, clish_sym_free);
 
 	/* Create userdata storage */
-	this->udata = lub_list_new(clish_udata_compare);
-	assert(this->udata);
+	this->udata = lub_list_new(clish_udata_compare, clish_udata_delete);
 
 	/* Hooks */
 	for (i = 0; i < CLISH_SYM_TYPE_MAX; i++) {
@@ -77,6 +72,7 @@ static void clish_shell_init(clish_shell_t * this,
 	this->dryrun = BOOL_FALSE; /* Disable dry-run by default */
 	this->user = lub_db_getpwuid(getuid()); /* Get user information */
 	this->default_plugin = BOOL_TRUE; /* Load default plugin by default */
+	this->canon_out = BOOL_FALSE; /* A canonical output is needed in special cases only */
 
 	/* Create template (string) for FIFO name generation */
 	snprintf(template, sizeof(template),
@@ -89,8 +85,8 @@ static void clish_shell_init(clish_shell_t * this,
 	tmp_ptype = clish_shell_find_create_ptype(this,
 		"__ptype_ARGS",
 		"Arguments", "[^\\\\]+",
-		CLISH_PTYPE_REGEXP,
-		CLISH_PTYPE_NONE);
+		CLISH_PTYPE_METHOD_REGEXP,
+		CLISH_PTYPE_PRE_NONE);
 	assert(tmp_ptype);
 
 	/* Push non-NULL istream */
@@ -101,34 +97,17 @@ static void clish_shell_init(clish_shell_t * this,
 /*--------------------------------------------------------- */
 static void clish_shell_fini(clish_shell_t *this)
 {
-	clish_view_t *view;
-	clish_ptype_t *ptype;
 	clish_var_t *var;
-	unsigned i;
-	lub_list_node_t *iter;
+	unsigned int i;
 
 	/* Free all loaded plugins */
-	while ((iter = lub_list__get_head(this->plugins))) {
-		/* Remove the symbol from the list */
-		lub_list_del(this->plugins, iter);
-		/* Free the instance */
-		clish_plugin_free((clish_plugin_t *)lub_list_node__get_data(iter),
-			(void *)this);
-		lub_list_node_free(iter);
-	}
-	lub_list_free(this->plugins);
+	lub_list_free_all(this->plugins);
 
-	/* delete each VIEW held  */
-	while ((view = lub_bintree_findfirst(&this->view_tree))) {
-		lub_bintree_remove(&this->view_tree, view);
-		clish_view_delete(view);
-	}
+	/* Delete each VIEW  */
+	lub_list_free_all(this->view_tree);
 
-	/* delete each PTYPE held  */
-	while ((ptype = lub_bintree_findfirst(&this->ptype_tree))) {
-		lub_bintree_remove(&this->ptype_tree, ptype);
-		clish_ptype_delete(ptype);
-	}
+	/* Delete each PTYPE  */
+	lub_list_free_all(this->ptype_tree);
 
 	/* delete each VAR held  */
 	while ((var = lub_bintree_findfirst(&this->var_tree))) {
@@ -144,24 +123,10 @@ static void clish_shell_fini(clish_shell_t *this)
 	}
 
 	/* Free symbol list */
-	while ((iter = lub_list__get_head(this->syms))) {
-		/* Remove the symbol from the list */
-		lub_list_del(this->syms, iter);
-		/* Free the instance */
-		clish_sym_free((clish_sym_t *)lub_list_node__get_data(iter));
-		lub_list_node_free(iter);
-	}
-	lub_list_free(this->syms);
+	lub_list_free_all(this->syms);
 
 	/* Free user data storage */
-	while ((iter = lub_list__get_head(this->udata))) {
-		/* Remove the symbol from the list */
-		lub_list_del(this->udata, iter);
-		/* Free the instance */
-		clish_udata_free((clish_udata_t *)lub_list_node__get_data(iter));
-		lub_list_node_free(iter);
-	}
-	lub_list_free(this->udata);
+	lub_list_free_all(this->udata);
 
 	/* free the textual details */
 	lub_string_free(this->overview);
@@ -214,10 +179,4 @@ void clish_shell_delete(clish_shell_t *this)
 	free(this);
 }
 
-/*--------------------------------------------------------- */
-struct passwd *clish_shell__get_user(clish_shell_t * this)
-{
-	return this->user;
-}
-
-/*-------------------------------------------------------- */
+CLISH_GET(shell, struct passwd *, user);
