@@ -304,8 +304,7 @@ static int process_clish_module(clish_shell_t *shell, clish_xmlnode_t *element,
 {
 	/* Create the global view */
 	if (!shell->global)
-		shell->global = clish_shell_find_create_view(shell,
-			"__view_global", "");
+		shell->global = clish_shell_find_create_view(shell, "__view_global");
 
 	parent = parent; /* Happy compiler */
 
@@ -332,7 +331,10 @@ static int process_view(clish_shell_t *shell, clish_xmlnode_t *element,
 	}
 
 	/* re-use a view if it already exists */
-	view = clish_shell_find_create_view(shell, name, prompt);
+	view = clish_shell_find_create_view(shell, name);
+
+	if (prompt)
+		clish_view__set_prompt(view, prompt);
 
 	if (depth && (lub_ctype_isdigit(*depth))) {
 		unsigned int res = 0;
@@ -373,36 +375,46 @@ static int process_ptype(clish_shell_t *shell, clish_xmlnode_t *element,
 	clish_ptype_method_e method;
 	clish_ptype_preprocess_e preprocess;
 	int res = -1;
+	clish_ptype_t *ptype;
 
 	char *name = clish_xmlnode_fetch_attr(element, "name");
 	char *help = clish_xmlnode_fetch_attr(element, "help");
 	char *pattern = clish_xmlnode_fetch_attr(element, "pattern");
 	char *method_name = clish_xmlnode_fetch_attr(element, "method");
 	char *preprocess_name =	clish_xmlnode_fetch_attr(element, "preprocess");
+	char *completion = clish_xmlnode_fetch_attr(element, "completion");
 
 	/* Check syntax */
 	if (!name) {
 		fprintf(stderr, CLISH_XML_ERROR_ATTR("name"));
 		goto error;
 	}
-	if (!pattern) {
+
+	method = clish_ptype_method_resolve(method_name);
+	if (CLISH_PTYPE_METHOD_MAX == method) {
+		fprintf(stderr, CLISH_XML_ERROR_ATTR("method"));
+		goto error;
+	}
+	if ((method != CLISH_PTYPE_METHOD_CODE) && !pattern) {
 		fprintf(stderr, CLISH_XML_ERROR_ATTR("pattern"));
 		goto error;
 	}
 
-	method = clish_ptype_method_resolve(method_name);
-
 	preprocess = clish_ptype_preprocess_resolve(preprocess_name);
-	clish_shell_find_create_ptype(shell,
+	ptype = clish_shell_find_create_ptype(shell,
 		name, help, pattern, method, preprocess);
 
-	res = 0;
+	if (completion)
+		clish_ptype__set_completion(ptype, completion);
+
+	res = process_children(shell, element, ptype);
 error:
 	clish_xml_release(name);
 	clish_xml_release(help);
 	clish_xml_release(pattern);
 	clish_xml_release(method_name);
 	clish_xml_release(preprocess_name);
+	clish_xml_release(completion);
 
 	parent = parent; /* Happy compiler */
 
@@ -468,9 +480,12 @@ static int process_command(clish_shell_t *shell, clish_xmlnode_t *element,
 	char *escape_chars = clish_xmlnode_fetch_attr(element, "escape_chars");
 	char *args_name = clish_xmlnode_fetch_attr(element, "args");
 	char *args_help = clish_xmlnode_fetch_attr(element, "args_help");
+	char *ref = clish_xmlnode_fetch_attr(element, "ref");
+#ifdef LEGACY
 	char *lock = clish_xmlnode_fetch_attr(element, "lock");
 	char *interrupt = clish_xmlnode_fetch_attr(element, "interrupt");
-	char *ref = clish_xmlnode_fetch_attr(element, "ref");
+	clish_action_t *action;
+#endif
 
 	/* Check syntax */
 	if (!name) {
@@ -540,17 +555,25 @@ static int process_command(clish_shell_t *shell, clish_xmlnode_t *element,
 	if (viewid)
 		clish_command__set_viewid(cmd, viewid);
 
-	/* lock field */
-	if (lock && lub_string_nocasecmp(lock, "false") == 0)
-		clish_command__set_lock(cmd, BOOL_FALSE);
-	else
-		clish_command__set_lock(cmd, BOOL_TRUE);
+#ifdef LEGACY
+	action = clish_command__get_action(cmd);
 
-	/* interrupt field */
-	if (interrupt && lub_string_nocasecmp(interrupt, "true") == 0)
-		clish_command__set_interrupt(cmd, BOOL_TRUE);
-	else
-		clish_command__set_interrupt(cmd, BOOL_FALSE);
+	/* lock */
+	if (lock) { // Don't change anything if lock is not specified
+		if (lub_string_nocasecmp(lock, "false") == 0)
+			clish_action__set_lock(action, BOOL_FALSE);
+		else
+			clish_action__set_lock(action, BOOL_TRUE);
+	}
+
+	/* interrupt */
+	if (interrupt) { // Don't change anything if lock is not specified
+		if (lub_string_nocasecmp(interrupt, "true") == 0)
+			clish_action__set_interrupt(action, BOOL_TRUE);
+		else
+			clish_action__set_interrupt(action, BOOL_FALSE);
+	}
+#endif
 
 	if (access)
 		clish_command__set_access(cmd, access);
@@ -566,9 +589,11 @@ error:
 	clish_xml_release(escape_chars);
 	clish_xml_release(args_name);
 	clish_xml_release(args_help);
+	clish_xml_release(ref);
+#ifdef LEGACY
 	clish_xml_release(lock);
 	clish_xml_release(interrupt);
-	clish_xml_release(ref);
+#endif
 
 	return res;
 }
@@ -586,10 +611,14 @@ static int process_startup(clish_shell_t *shell, clish_xmlnode_t *element,
 	char *default_shebang =
 		clish_xmlnode_fetch_attr(element, "default_shebang");
 	char *timeout = clish_xmlnode_fetch_attr(element, "timeout");
-	char *lock = clish_xmlnode_fetch_attr(element, "lock");
-	char *interrupt = clish_xmlnode_fetch_attr(element, "interrupt");
 	char *default_plugin = clish_xmlnode_fetch_attr(element,
 		"default_plugin");
+#ifdef LEGACY
+	char *lock = clish_xmlnode_fetch_attr(element, "lock");
+	char *interrupt = clish_xmlnode_fetch_attr(element, "interrupt");
+	clish_action_t *action;
+#endif
+
 
 	/* Check syntax */
 	if (!view) {
@@ -603,7 +632,7 @@ static int process_startup(clish_shell_t *shell, clish_xmlnode_t *element,
 
 	/* create a command with NULL help */
 	cmd = clish_view_new_command(v, "startup", NULL);
-	clish_command__set_lock(cmd, BOOL_FALSE);
+	clish_command__set_internal(cmd, BOOL_TRUE);
 
 	/* reference the next view */
 	clish_command__set_viewname(cmd, view);
@@ -618,20 +647,28 @@ static int process_startup(clish_shell_t *shell, clish_xmlnode_t *element,
 	if (timeout) {
 		unsigned int to = 0;
 		lub_conv_atoui(timeout, &to, 0);
-		clish_shell__set_timeout(shell, to);
+		clish_shell__set_idle_timeout(shell, to);
 	}
 
-	/* lock field */
-	if (lock && lub_string_nocasecmp(lock, "false") == 0)
-		clish_command__set_lock(cmd, BOOL_FALSE);
-	else
-		clish_command__set_lock(cmd, BOOL_TRUE);
+#ifdef LEGACY
+	action = clish_command__get_action(cmd);
 
-	/* interrupt field */
-	if (interrupt && lub_string_nocasecmp(interrupt, "true") == 0)
-		clish_command__set_interrupt(cmd, BOOL_TRUE);
-	else
-		clish_command__set_interrupt(cmd, BOOL_FALSE);
+	/* lock */
+	if (lock) { // Don't change anything if lock is not specified
+		if (lub_string_nocasecmp(lock, "false") == 0)
+			clish_action__set_lock(action, BOOL_FALSE);
+		else
+			clish_action__set_lock(action, BOOL_TRUE);
+	}
+
+	/* interrupt */
+	if (interrupt) { // Don't change anything if lock is not specified
+		if (lub_string_nocasecmp(interrupt, "true") == 0)
+			clish_action__set_interrupt(action, BOOL_TRUE);
+		else
+			clish_action__set_interrupt(action, BOOL_FALSE);
+	}
+#endif
 
 	/* If we need the default plugin */
 	if (default_plugin && (0 == strcmp(default_plugin, "false")))
@@ -646,8 +683,10 @@ error:
 	clish_xml_release(viewid);
 	clish_xml_release(default_shebang);
 	clish_xml_release(timeout);
+#ifdef LEGACY
 	clish_xml_release(lock);
 	clish_xml_release(interrupt);
+#endif
 
 	return res;
 }
@@ -724,12 +763,9 @@ static int process_param(clish_shell_t *shell, clish_xmlnode_t *element,
 		 * name of ptype is hardcoded. It's not good but
 		 * it's only the service ptype.
 		 */
-		tmp = (clish_ptype_t *)lub_bintree_find(
-			&shell->ptype_tree, ptype_name);
-		if (!tmp)
-			tmp = clish_shell_find_create_ptype(shell,
-				ptype_name, "Option", "[^\\\\]+",
-				CLISH_PTYPE_REGEXP, CLISH_PTYPE_NONE);
+		tmp = clish_shell_find_create_ptype(shell,
+			ptype_name, "Option", "[^\\\\]+",
+			CLISH_PTYPE_METHOD_REGEXP, CLISH_PTYPE_PRE_NONE);
 		assert(tmp);
 		lub_string_cat(&str, "_prefix_");
 		lub_string_cat(&str, name);
@@ -755,7 +791,7 @@ static int process_param(clish_shell_t *shell, clish_xmlnode_t *element,
 	}
 
 	if (defval)
-		clish_param__set_default(param, defval);
+		clish_param__set_defval(param, defval);
 
 	if (hidden && lub_string_nocasecmp(hidden, "true") == 0)
 		clish_param__set_hidden(param, BOOL_TRUE);
@@ -835,8 +871,13 @@ static int process_action(clish_shell_t *shell, clish_xmlnode_t *element,
 	void *parent)
 {
 	clish_action_t *action = NULL;
+
 	char *builtin = clish_xmlnode_fetch_attr(element, "builtin");
 	char *shebang = clish_xmlnode_fetch_attr(element, "shebang");
+	char *lock = clish_xmlnode_fetch_attr(element, "lock");
+	char *interrupt = clish_xmlnode_fetch_attr(element, "interrupt");
+	char *interactive = clish_xmlnode_fetch_attr(element, "interactive");
+
 	clish_xmlnode_t *pelement = clish_xmlnode_parent(element);
 	char *pname = clish_xmlnode_get_all_name(pelement);
 	char *text;
@@ -844,6 +885,8 @@ static int process_action(clish_shell_t *shell, clish_xmlnode_t *element,
 
 	if (pname && lub_string_nocasecmp(pname, "VAR") == 0)
 		action = clish_var__get_action((clish_var_t *)parent);
+	else if (pname && lub_string_nocasecmp(pname, "PTYPE") == 0)
+		action = clish_ptype__get_action((clish_ptype_t *)parent);
 	else
 		action = clish_command__get_action((clish_command_t *)parent);
 
@@ -869,8 +912,19 @@ static int process_action(clish_shell_t *shell, clish_xmlnode_t *element,
 	if (shebang)
 		clish_action__set_shebang(action, shebang);
 
+	/* lock */
+	if (lock && lub_string_nocasecmp(lock, "false") == 0)
+		clish_action__set_lock(action, BOOL_FALSE);
+
+	/* interrupt */
+	if (interrupt && lub_string_nocasecmp(interrupt, "true") == 0)
+		clish_action__set_interrupt(action, BOOL_TRUE);
+
 	clish_xml_release(builtin);
 	clish_xml_release(shebang);
+	clish_xml_release(lock);
+	clish_xml_release(interrupt);
+	clish_xml_release(interactive);
 
 	return 0;
 }
@@ -1119,7 +1173,10 @@ static int process_wdog(clish_shell_t *shell,
 
 	/* Create a command with NULL help */
 	cmd = clish_view_new_command(v, "watchdog", NULL);
-	clish_command__set_lock(cmd, BOOL_FALSE);
+#ifdef LEGACY
+	// Legacy watchdog has lockless ACTION
+	clish_action__set_lock(clish_command__get_action(cmd), BOOL_FALSE);
+#endif
 
 	/* Remember this command */
 	shell->wdog = cmd;
@@ -1185,8 +1242,7 @@ static int process_plugin(clish_shell_t *shell, clish_xmlnode_t *element,
 			CLISH_XML_ERROR_STR"PLUGIN %s duplication.\n", name);
 		goto error;
 	}
-	plugin = clish_plugin_new(name);
-	lub_list_add(shell->plugins, plugin);
+	plugin = clish_shell_create_plugin(shell, name);
 
 	if (alias && *alias)
 		clish_plugin__set_alias(plugin, alias);
